@@ -1,6 +1,6 @@
 /* ============================================================
-* QupZilla - WebKit based browser
-* Copyright (C) 2010-2016 David Rosca <nowrep@gmail.com>
+* QupZilla - Qt web browser
+* Copyright (C) 2010-2017 David Rosca <nowrep@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -97,7 +97,7 @@ bool AdBlockManager::block(QWebEngineUrlRequestInfo &request)
     const QString urlDomain = request.requestUrl().host().toLower();
     const QString urlScheme = request.requestUrl().scheme().toLower();
 
-    if (!isEnabled() || !canRunOnScheme(urlScheme))
+    if (!isEnabled() || !canRunOnScheme(urlScheme) || !canBeBlocked(request.firstPartyUrl()))
         return 0;
 
     bool res = false;
