@@ -1,6 +1,6 @@
 /* ============================================================
-* QupZilla - WebKit based browser
-* Copyright (C) 2010-2016 David Rosca <nowrep@gmail.com>
+* QupZilla - Qt web browser
+* Copyright (C) 2010-2017 David Rosca <nowrep@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -99,7 +99,6 @@ int ProfileManager::createProfile(const QString &profileName)
         return -2;
     }
 
-    dir.mkdir(profileName);
     dir.cd(profileName);
     QFile(QLatin1String(":data/browsedata.db")).copy(dir.filePath(QLatin1String("browsedata.db")));
     QFile(dir.filePath(QLatin1String("browsedata.db"))).setPermissions(QFile::ReadUser | QFile::WriteUser);
@@ -189,8 +188,7 @@ void ProfileManager::updateProfile(const QString &current, const QString &profil
     Updater::Version prof(profile);
 
     if (prof < Updater::Version("1.9.0")) {
-        std::cout << "QupZilla: Incompatible profile version detected (" << qPrintable(profile) << "), overwriting profile data..." << std::endl;
-        copyDataToProfile();
+        std::cout << "QupZilla: Using profile from QupZilla " << qPrintable(profile) << " is not supported!" << std::endl;
         return;
     }
 
@@ -199,6 +197,11 @@ void ProfileManager::updateProfile(const QString &current, const QString &profil
         return;
     }
 
+    // No change in 2.1
+    if (prof < Updater::Version("2.1.99")) {
+        return;
+    }
+
     // Nothing for now
 }
 
