@@ -1,6 +1,6 @@
 /* ============================================================
-* QupZilla - WebKit based browser
-* Copyright (C) 2010-2016  David Rosca <nowrep@gmail.com>
+* QupZilla - Qt web browser
+* Copyright (C) 2010-2017 David Rosca <nowrep@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -107,13 +107,13 @@ WebView *WebPage::view() const
     return static_cast<WebView*>(QWebEnginePage::view());
 }
 
-QVariant WebPage::execJavaScript(const QString &scriptSource, int timeout)
+QVariant WebPage::execJavaScript(const QString &scriptSource, quint32 worldId, int timeout)
 {
     QPointer<QEventLoop> loop = new QEventLoop;
     QVariant result;
     QTimer::singleShot(timeout, loop.data(), &QEventLoop::quit);
 
-    runJavaScript(scriptSource, [loop, &result](const QVariant &res) {
+    runJavaScript(scriptSource, worldId, [loop, &result](const QVariant &res) {
         if (loop && loop->isRunning()) {
             result = res;
             loop->quit();
@@ -126,9 +126,29 @@ QVariant WebPage::execJavaScript(const QString &scriptSource, int timeout)
     return result;
 }
 
-QPoint WebPage::mapToViewport(const QPoint &pos) const
+void WebPage::runJavaScript(const QString &scriptSource)
 {
-    return QPoint(pos.x() / zoomFactor(), pos.y() / zoomFactor());
+    return QWebEnginePage::runJavaScript(scriptSource);
+}
+
+void WebPage::runJavaScript(const QString &scriptSource, const QWebEngineCallback<const QVariant &> &resultCallback)
+{
+    return QWebEnginePage::runJavaScript(scriptSource, resultCallback);
+}
+
+void WebPage::runJavaScript(const QString &scriptSource, quint32 worldId)
+{
+    QWebEnginePage::runJavaScript(scriptSource, worldId);
+}
+
+void WebPage::runJavaScript(const QString &scriptSource, quint32 worldId, const QWebEngineCallback<const QVariant &> &resultCallback)
+{
+    QWebEnginePage::runJavaScript(scriptSource, worldId, resultCallback);
+}
+
+QPointF WebPage::mapToViewport(const QPointF &pos) const
+{
+    return QPointF(pos.x() / zoomFactor(), pos.y() / zoomFactor());
 }
 
 WebHitTestResult WebPage::hitTestContent(const QPoint &pos) const
@@ -138,7 +158,13 @@ WebHitTestResult WebPage::hitTestContent(const QPoint &pos) const
 
 void WebPage::scroll(int x, int y)
 {
-    runJavaScript(QSL("window.scrollTo(window.scrollX + %1, window.scrollY + %2)").arg(x).arg(y));
+    runJavaScript(QSL("window.scrollTo(window.scrollX + %1, window.scrollY + %2)").arg(x).arg(y), WebPage::SafeJsWorld);
+}
+
+void WebPage::setScrollPosition(const QPointF &pos)
+{
+    const QPointF v = mapToViewport(pos.toPoint());
+    runJavaScript(QSL("window.scrollTo(%1, %2)").arg(v.x()).arg(v.y()), WebPage::SafeJsWorld);
 }
 
 void WebPage::scheduleAdjustPage()
@@ -350,21 +376,18 @@ void WebPage::renderProcessTerminated(QWebEnginePage::RenderProcessTerminationSt
     if (terminationStatus == NormalTerminationStatus)
         return;
 
-#if QT_VERSION != QT_VERSION_CHECK(5, 7, 0) // Crashes with QtWebEngine 5.7.0
     QTimer::singleShot(0, this, [this]() {
         QString page = QzTools::readAllFileContents(":html/tabcrash.html");
         page.replace(QL1S("%IMAGE%"), QzTools::pixmapToDataUrl(IconProvider::standardIcon(QStyle::SP_MessageBoxWarning).pixmap(45)).toString());
-        page.replace(QL1S("%FAVICON%"), QzTools::pixmapToDataUrl(IconProvider::standardIcon(QStyle::SP_MessageBoxWarning).pixmap(16)).toString());
-        page.replace(QL1S("%BOX-BORDER%"), QLatin1String("qrc:html/box-border.png"));
         page.replace(QL1S("%TITLE%"), tr("Failed loading page"));
         page.replace(QL1S("%HEADING%"), tr("Failed loading page"));
         page.replace(QL1S("%LI-1%"), tr("Something went wrong while loading this page."));
         page.replace(QL1S("%LI-2%"), tr("Try reloading the page or closing some tabs to make more memory available."));
         page.replace(QL1S("%RELOAD-PAGE%"), tr("Reload page"));
         page = QzTools::applyDirectionToPage(page);
+        load(url()); // Workaround for QtWebEngine crash
         setHtml(page.toUtf8(), url());
     });
-#endif
 }
 
 bool WebPage::acceptNavigationRequest(const QUrl &url, QWebEnginePage::NavigationType type, bool isMainFrame)
@@ -433,12 +456,12 @@ void WebPage::cleanBlockedObjects()
     // Apply global element hiding rules
     const QString elementHiding = manager->elementHidingRules(url());
     if (!elementHiding.isEmpty())
-        runJavaScript(Scripts::setCss(elementHiding));
+        runJavaScript(Scripts::setCss(elementHiding), WebPage::SafeJsWorld);
 
     // Apply domain-specific element hiding rules
     const QString siteElementHiding = manager->elementHidingRulesForDomain(url());
     if (!siteElementHiding.isEmpty())
-        runJavaScript(Scripts::setCss(siteElementHiding));
+        runJavaScript(Scripts::setCss(siteElementHiding), WebPage::SafeJsWorld);
 }
 
 bool WebPage::javaScriptPrompt(const QUrl &securityOrigin, const QString &msg, const QString &defaultValue, QString* result)
@@ -589,6 +612,13 @@ QWebEnginePage* WebPage::createWindow(QWebEnginePage::WebWindowType type)
     TabbedWebView *tView = qobject_cast<TabbedWebView*>(view());
     BrowserWindow *window = tView ? tView->browserWindow() : mApp->getWindow();
 
+    auto createTab = [=](Qz::NewTabPositionFlags pos) {
+        int index = window->tabWidget()->addView(QUrl(), pos);
+        TabbedWebView* view = window->weView(index);
+        view->setPage(new WebPage);
+        return view->page();
+    };
+
     switch (type) {
     case QWebEnginePage::WebBrowserWindow: {
         BrowserWindow *window = mApp->createWindow(Qz::BW_NewWindow);
@@ -608,12 +638,11 @@ QWebEnginePage* WebPage::createWindow(QWebEnginePage::WebWindowType type)
         }
         // else fallthrough
 
-    case QWebEnginePage::WebBrowserTab: {
-        int index = window->tabWidget()->addView(QUrl(), Qz::NT_CleanSelectedTab);
-        TabbedWebView* view = window->weView(index);
-        view->setPage(new WebPage);
-        return view->page();
-    }
+    case QWebEnginePage::WebBrowserTab:
+        return createTab(Qz::NT_CleanSelectedTab);
+
+    case QWebEnginePage::WebBrowserBackgroundTab:
+        return createTab(Qz::NT_CleanNotSelectedTab);
 
     default:
         break;
